@@ -9,10 +9,10 @@
 #
 # Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along with
-# Gaupol.  If not, see <http://www.gnu.org/licenses/>.
+# Gaupol. If not, see <http://www.gnu.org/licenses/>.
 
 """Extension delegates of Application."""
 
@@ -31,4 +31,4 @@ from .update   import UpdateAgent
 from .util     import UtilityAgent
 from .view     import ViewAgent
 
-__all__ = [x for x in dir() if x.endswith("Agent")]
+__all__ = tuple(x for x in dir() if x.endswith("Agent"))
