@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2007 Osmo Salomaa
+# Copyright (C) 2005-2008 Osmo Salomaa
 #
 # This file is part of Gaupol.
 #
@@ -9,10 +9,10 @@
 #
 # Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along with
-# Gaupol.  If not, see <http://www.gnu.org/licenses/>.
+# Gaupol. If not, see <http://www.gnu.org/licenses/>.
 
 """Previewing subtitles with a video player."""
 
@@ -21,8 +21,6 @@ from __future__ import with_statement
 import gaupol.gtk
 import gobject
 import gtk
-import os
-import sys
 import tempfile
 _ = gaupol.i18n._
 
@@ -33,23 +31,22 @@ class PreviewAgent(gaupol.Delegate):
 
     # pylint: disable-msg=E0203,W0201
 
-    def _check_process_state(self, process, output_path):
+    def _check_process_state(self, process, output_path, command):
         """Check if the process has terminated or not."""
 
-        if process.poll() is not None:
-            self._handle_output(process, output_path)
-            return False
-        return True
+        if process.poll() is None: return True
+        self._handle_output(process, output_path, command)
+        return False # to not check again.
 
-    @gaupol.gtk.util.asserted_return
-    def _handle_output(self, process, output_path):
+    def _handle_output(self, process, output_path, command):
         """Handle the output of finished preview process."""
 
         with open(output_path, "r") as fobj:
             output = fobj.read()
+        output = "$ %s\n\n%s" % (command, output)
         gaupol.temp.remove(output_path)
         self.output_window.set_output(output)
-        assert process.returncode != 0
+        if process.returncode == 0: return
         dialog = gaupol.gtk.PreviewErrorDialog(self.window, output)
         self.flash_dialog(dialog)
 
@@ -75,6 +72,15 @@ class PreviewAgent(gaupol.Delegate):
         dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
         self.flash_dialog(dialog)
 
+    def _show_process_error_dialog(self, message):
+        """Show an error dialog after failing to launch video player."""
+
+        title = _("Failed to launch video player")
+        message = _("%s.") % message
+        dialog = gaupol.gtk.ErrorDialog(self.window, title, message)
+        dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
+        self.flash_dialog(dialog)
+
     def on_preview_activate(self, *args):
         """Preview from selected position with a video player."""
 
@@ -82,30 +88,33 @@ class PreviewAgent(gaupol.Delegate):
         rows = page.view.get_selected_rows()
         row = (rows[0] if rows else 0)
         time = page.project.subtitles[row].start_time
-        col = gaupol.gtk.COLUMN.MAIN_TEXT
-        if page.view.get_focus()[1] == gaupol.gtk.COLUMN.TRAN_TEXT:
-            col = gaupol.gtk.COLUMN.TRAN_TEXT
-        doc = gaupol.gtk.util.text_column_to_document(col)
+        col = page.view.get_focus()[1]
+        if col == page.view.columns.TRAN_TEXT:
+            doc = gaupol.documents.TRAN
+        else: # Any other column previews the main file.
+            doc = gaupol.documents.MAIN
         self.preview(page, time, doc)
 
-    @gaupol.gtk.util.asserted_return
     def preview(self, page, time, doc, path=None):
         """Preview from time with a video player."""
 
-        try:
-            command = gaupol.gtk.conf.preview.video_player.command
-            if gaupol.gtk.conf.preview.use_custom:
-                command = gaupol.gtk.conf.preview.custom_command
-            offset = gaupol.gtk.conf.preview.offset
-            output = page.project.preview(time, doc, command, offset, path)
-            process, command, output_path = output
+        command = gaupol.gtk.util.get_preview_command()
+        offset = gaupol.gtk.conf.preview.offset
+        encoding = ("utf_8" if gaupol.gtk.conf.preview.force_utf_8 else None)
+        preview = page.project.preview
+        args = (time, doc, command, offset, path, encoding)
+        try: process, command, output_path = preview(*args)
+        except gaupol.ProcessError, message:
+            return self._show_process_error_dialog(message)
         except (IOError, OSError), (no, message):
             return self._show_io_error_dialog(message)
         except UnicodeError:
             return self._show_encoding_error_dialog()
+        # 'gobject.child_watch_add' does not appear to work on Windows,
+        # so let's watch the process by polling it at regular intervals.
         function = self._check_process_state
-        args = (process, output_path)
-        gobject.timeout_add(500, function, *args)
+        args = (process, output_path, command)
+        gobject.timeout_add(1000, function, *args)
 
     def preview_changes(self, page, row, doc, method, args=None, kwargs=None):
         """Preview changes caused by method with a video player."""
@@ -113,9 +122,7 @@ class PreviewAgent(gaupol.Delegate):
         subtitles = [x.copy() for x in page.project.subtitles]
         framerate = page.project.framerate
         blocked = page.project.block_all()
-        kwargs = kwargs or {}
-        kwargs["register"] = None
-        method(*(args or ()), **kwargs)
+        method(register=None, *(args or ()), **(kwargs or {}))
         path = page.project.get_temp_file_path(doc)
         time = page.project.subtitles[row].start_time
         page.project.unblock_all(blocked)
