@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2007 Osmo Salomaa
+# Copyright (C) 2005-2008 Osmo Salomaa
 #
 # This file is part of Gaupol.
 #
@@ -9,38 +9,38 @@
 #
 # Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along with
-# Gaupol.  If not, see <http://www.gnu.org/licenses/>.
+# Gaupol. If not, see <http://www.gnu.org/licenses/>.
 
 """Dialog for configuring spell-check."""
 
 import gaupol.gtk
-import gobject
 import gtk
 
-from .glade import GladeDialog
+__all__ = ("LanguageDialog",)
 
 
-class LanguageDialog(GladeDialog):
+class LanguageDialog(gaupol.gtk.GladeDialog):
 
     """Dialog for configuring spell-check."""
 
     __metaclass__ = gaupol.Contractual
 
     def __init___require(self, parent):
-        assert gaupol.gtk.util.enchant_available()
+        assert gaupol.util.enchant_available()
 
     def __init__(self, parent):
 
-        GladeDialog.__init__(self, "language-dialog")
+        gaupol.gtk.GladeDialog.__init__(self, "language.glade")
         get_widget = self._glade_xml.get_widget
         self._all_radio = get_widget("all_radio")
         self._current_radio = get_widget("current_radio")
         self._main_radio = get_widget("main_radio")
         self._tran_radio = get_widget("tran_radio")
         self._tree_view = get_widget("tree_view")
+        self.conf = gaupol.gtk.conf.spell_check
 
         self._init_tree_view()
         self._init_values()
@@ -52,9 +52,9 @@ class LanguageDialog(GladeDialog):
     def _init_signal_handlers(self):
         """Initialize signal handlers."""
 
-        on_column_toggled = lambda x, self: self._save_column()
-        self._main_radio.connect("toggled", on_column_toggled, self)
-        self._tran_radio.connect("toggled", on_column_toggled, self)
+        on_field_toggled = lambda x, self: self._save_field()
+        self._main_radio.connect("toggled", on_field_toggled, self)
+        self._tran_radio.connect("toggled", on_field_toggled, self)
         on_target_toggled = lambda x, self: self._save_target()
         self._all_radio.connect("toggled", on_target_toggled, self)
         self._current_radio.connect("toggled", on_target_toggled, self)
@@ -66,16 +66,15 @@ class LanguageDialog(GladeDialog):
         """Initialize widget sizes."""
 
         width, height = gaupol.gtk.util.get_tree_view_size(self._tree_view)
-        width = width + 42 + gaupol.gtk.EXTRA
-        height = height + 259 + gaupol.gtk.EXTRA
-        gaupol.gtk.util.resize_dialog(self, width, height, 0.5)
+        width = min(width, int(0.5 * gtk.gdk.screen_width()))
+        self._tree_view.set_size_request(width + gaupol.gtk.EXTRA, -1)
 
     def _init_tree_view(self):
         """Initialize the tree view."""
 
         selection = self._tree_view.get_selection()
         selection.set_mode(gtk.SELECTION_SINGLE)
-        store = gtk.ListStore(*(gobject.TYPE_STRING,) * 2)
+        store = gtk.ListStore(str, str)
         self._populate_store(store)
         store.set_sort_column_id(1, gtk.SORT_ASCENDING)
         self._tree_view.set_model(store)
@@ -91,50 +90,47 @@ class LanguageDialog(GladeDialog):
         store = self._tree_view.get_model()
         selection = self._tree_view.get_selection()
         for i in range(len(store)):
-            if store[i][0] == gaupol.gtk.conf.spell_check.language:
+            if store[i][0] == self.conf.language:
                 selection.select_path(i)
-        col = gaupol.gtk.conf.spell_check.column
-        self._main_radio.set_active(col == gaupol.gtk.COLUMN.MAIN_TEXT)
-        self._tran_radio.set_active(col == gaupol.gtk.COLUMN.TRAN_TEXT)
-        target = gaupol.gtk.conf.spell_check.target
-        self._all_radio.set_active(target == gaupol.gtk.TARGET.ALL)
-        self._current_radio.set_active(target == gaupol.gtk.TARGET.CURRENT)
+        fields = gaupol.gtk.fields
+        self._main_radio.set_active(self.conf.field == fields.MAIN_TEXT)
+        self._tran_radio.set_active(self.conf.field == fields.TRAN_TEXT)
+        targets = gaupol.gtk.targets
+        self._all_radio.set_active(self.conf.target == targets.ALL)
+        self._current_radio.set_active(self.conf.target == targets.CURRENT)
 
-    @gaupol.gtk.util.asserted_return
     def _on_tree_view_selection_changed(self, selection):
         """Save the active language."""
 
         store, itr = selection.get_selected()
-        assert itr is not None
+        if itr is None: return
         value = store.get_value(itr, 0)
-        gaupol.gtk.conf.spell_check.language = value
+        self.conf.language = value
 
     def _populate_store(self, store):
         """Add all available languages to the list store."""
 
         import enchant
-        @gaupol.gtk.util.silent(enchant.Error)
-        def append(locale):
-            enchant.Dict(locale)
+        for locale in gaupol.locales.get_all():
+            try: enchant.Dict(locale).check("1")
+            except enchant.Error: continue
             name = gaupol.locales.code_to_name(locale)
-            store.append([locale, name])
-        for locale in gaupol.locales.locales:
-            append(locale)
+            store.append((locale, name))
 
-    def _save_column(self):
-        """Save the active column."""
+    def _save_field(self):
+        """Save the active field."""
 
         if self._main_radio.get_active():
-            col = gaupol.gtk.COLUMN.MAIN_TEXT
+            field = gaupol.gtk.fields.MAIN_TEXT
         elif self._tran_radio.get_active():
-            col = gaupol.gtk.COLUMN.TRAN_TEXT
-        gaupol.gtk.conf.spell_check.column = col
+            field = gaupol.gtk.fields.TRAN_TEXT
+        self.conf.field = field
 
     def _save_target(self):
         """Save the active target."""
 
         if self._current_radio.get_active():
-            target = gaupol.gtk.TARGET.CURRENT
+            target = gaupol.gtk.targets.CURRENT
         elif self._all_radio.get_active():
-            target = gaupol.gtk.TARGET.ALL
-        gaupol.gtk.conf.spell_check.target = target
+            target = gaupol.gtk.targets.ALL
+        self.conf.target = target
