@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2007 Osmo Salomaa
+# Copyright (C) 2005-2008 Osmo Salomaa
 #
 # This file is part of Gaupol.
 #
@@ -9,21 +9,19 @@
 #
 # Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along with
-# Gaupol.  If not, see <http://www.gnu.org/licenses/>.
+# Gaupol. If not, see <http://www.gnu.org/licenses/>.
 
 """Dialog for editing preferences."""
 
 import gaupol.gtk
-import gobject
 import gtk
 import pango
 _ = gaupol.i18n._
 
-from .encoding import EncodingDialog
-from .glade import GladeDialog
+__all__ = ("PreferencesDialog",)
 
 
 class _EditorPage(gaupol.Delegate):
@@ -41,17 +39,18 @@ class _EditorPage(gaupol.Delegate):
         self._length_combo = get_widget("editor_length_combo")
         self._length_edit_check = get_widget("editor_length_edit_check")
         self._length_hbox = get_widget("editor_length_hbox")
+        self.conf = gaupol.gtk.conf.editor
 
         self._init_length_combo()
         self._init_values()
         self._init_signal_handlers()
 
     def _get_custom_font(self):
-        """Get custom font as string."""
+        """Return custom font as string."""
 
         context = gtk.Label().get_pango_context()
         font_desc = context.get_font_description()
-        font = gaupol.gtk.conf.editor.custom_font
+        font = self.conf.custom_font
         custom_font_desc = pango.FontDescription(font)
         font_desc.merge(custom_font_desc, True)
         return font_desc.to_string()
@@ -60,66 +59,65 @@ class _EditorPage(gaupol.Delegate):
         """Initialize the line length combo box."""
 
         store = self._length_combo.get_model()
-        for name in gaupol.gtk.LENGTH_UNIT.labels:
-            store.append([name])
+        for label in (x.label for x in gaupol.gtk.length_units):
+            store.append((label,))
 
     def _init_signal_handlers(self):
         """Initialize signal handlers."""
 
-        gaupol.gtk.util.connect(self, "_default_font_check", "toggled")
-        gaupol.gtk.util.connect(self, "_font_button", "font-set")
-        gaupol.gtk.util.connect(self, "_length_cell_check", "toggled")
-        gaupol.gtk.util.connect(self, "_length_combo", "changed")
-        gaupol.gtk.util.connect(self, "_length_edit_check", "toggled")
+        gaupol.util.connect(self, "_default_font_check", "toggled")
+        gaupol.util.connect(self, "_font_button", "font-set")
+        gaupol.util.connect(self, "_length_cell_check", "toggled")
+        gaupol.util.connect(self, "_length_combo", "changed")
+        gaupol.util.connect(self, "_length_edit_check", "toggled")
 
     def _init_values(self):
         """Initialize default values for widgets."""
 
-        use_custom = gaupol.gtk.conf.editor.use_custom_font
+        use_custom = self.conf.use_custom_font
         self._default_font_check.set_active(not use_custom)
         self._font_hbox.set_sensitive(use_custom)
         self._font_button.set_font_name(self._get_custom_font())
 
-        cell = gaupol.gtk.conf.editor.show_lengths_cell
-        edit = gaupol.gtk.conf.editor.show_lengths_edit
+        cell = self.conf.show_lengths_cell
+        edit = self.conf.show_lengths_edit
         self._length_hbox.set_sensitive(cell or edit)
         self._length_cell_check.set_active(cell)
         self._length_edit_check.set_active(edit)
-        self._length_combo.set_active(gaupol.gtk.conf.editor.length_unit)
+        self._length_combo.set_active(self.conf.length_unit)
 
     def _on_default_font_check_toggled(self, check_button):
         """Save the default font usage."""
 
         use_custom = not check_button.get_active()
-        gaupol.gtk.conf.editor.use_custom_font = use_custom
+        self.conf.use_custom_font = use_custom
         self._font_hbox.set_sensitive(use_custom)
 
     def _on_font_button_font_set(self, font_button):
         """Save the custom font."""
 
-        gaupol.gtk.conf.editor.custom_font = font_button.get_font_name()
+        self.conf.custom_font = font_button.get_font_name()
 
     def _on_length_cell_check_toggled(self, check_button):
         """Save the line length showage on cells."""
 
-        gaupol.gtk.conf.editor.show_lengths_cell = check_button.get_active()
-        cell = gaupol.gtk.conf.editor.show_lengths_cell
-        edit = gaupol.gtk.conf.editor.show_lengths_edit
+        self.conf.show_lengths_cell = check_button.get_active()
+        cell = self.conf.show_lengths_cell
+        edit = self.conf.show_lengths_edit
         self._length_hbox.set_sensitive(cell or edit)
 
     def _on_length_combo_changed(self, combo_box):
         """Save the line length unit."""
 
         index = combo_box.get_active()
-        length_unit = gaupol.gtk.LENGTH_UNIT.members[index]
-        gaupol.gtk.conf.editor.length_unit = length_unit
+        self.conf.length_unit = gaupol.gtk.length_units[index]
 
     def _on_length_edit_check_toggled(self, check_button):
         """Save the line length showage on text views."""
 
-        gaupol.gtk.conf.editor.show_lengths_edit = check_button.get_active()
-        cell = gaupol.gtk.conf.editor.show_lengths_cell
-        edit = gaupol.gtk.conf.editor.show_lengths_edit
+        self.conf.show_lengths_edit = check_button.get_active()
+        cell = self.conf.show_lengths_cell
+        edit = self.conf.show_lengths_edit
         self._length_hbox.set_sensitive(cell or edit)
 
 
@@ -127,6 +125,8 @@ class _FilePage(gaupol.Delegate):
 
     """File preferences page."""
 
+    __metaclass__ = gaupol.Contractual
+
     def __init__(self, master):
 
         gaupol.Delegate.__init__(self, master)
@@ -138,29 +138,29 @@ class _FilePage(gaupol.Delegate):
         self._locale_check = get_widget("file_locale_check")
         self._remove_button = get_widget("file_remove_button")
         self._up_button = get_widget("file_up_button")
+        self.conf = gaupol.gtk.conf.encoding
 
         self._init_tree_view()
         self._init_values()
         self._init_signal_handlers()
 
-    @gaupol.gtk.util.asserted_return
     def _get_selected_row(self):
-        """Get the selected row in the tree view or None."""
+        """Return the selected row in the tree view or None."""
 
         selection = self._tree_view.get_selection()
         store, itr = selection.get_selected()
-        assert itr is not None
+        if itr is None: return
         return store.get_path(itr)[0]
 
     def _init_signal_handlers(self):
         """Initialize signal handlers."""
 
-        gaupol.gtk.util.connect(self, "_add_button", "clicked")
-        gaupol.gtk.util.connect(self, "_auto_check", "toggled")
-        gaupol.gtk.util.connect(self, "_down_button", "clicked")
-        gaupol.gtk.util.connect(self, "_locale_check", "toggled")
-        gaupol.gtk.util.connect(self, "_remove_button", "clicked")
-        gaupol.gtk.util.connect(self, "_up_button", "clicked")
+        gaupol.util.connect(self, "_add_button", "clicked")
+        gaupol.util.connect(self, "_auto_check", "toggled")
+        gaupol.util.connect(self, "_down_button", "clicked")
+        gaupol.util.connect(self, "_locale_check", "toggled")
+        gaupol.util.connect(self, "_remove_button", "clicked")
+        gaupol.util.connect(self, "_up_button", "clicked")
 
         update = lambda x, self: self._set_sensitivities()
         selection = self._tree_view.get_selection()
@@ -171,7 +171,7 @@ class _FilePage(gaupol.Delegate):
 
         selection = self._tree_view.get_selection()
         selection.set_mode(gtk.SELECTION_SINGLE)
-        store = gtk.ListStore(gobject.TYPE_STRING)
+        store = gtk.ListStore(str)
         self._tree_view.set_model(store)
         renderer = gtk.CellRendererText()
         column = gtk.TreeViewColumn("", renderer, text=0)
@@ -180,38 +180,45 @@ class _FilePage(gaupol.Delegate):
     def _init_values(self):
         """Initialize default values for widgets."""
 
-        self._auto_check.set_active(gaupol.gtk.conf.encoding.try_auto)
-        self._auto_check.set_sensitive(gaupol.gtk.util.chardet_available())
-        self._locale_check.set_active(gaupol.gtk.conf.encoding.try_locale)
+        self._auto_check.set_active(self.conf.try_auto)
+        self._auto_check.set_sensitive(gaupol.util.chardet_available())
+        self._locale_check.set_active(self.conf.try_locale)
         self._reload_tree_view()
 
-    @gaupol.gtk.util.asserted_return
     def _on_add_button_clicked(self, *args):
         """Add a new fallback encoding."""
 
-        dialog = EncodingDialog(self._dialog)
+        dialog = gaupol.gtk.EncodingDialog(self._dialog)
         response = self.run_dialog(dialog)
         encoding = dialog.get_encoding()
         dialog.destroy()
-        assert response == gtk.RESPONSE_OK
-        assert encoding is not None
-        assert encoding not in gaupol.gtk.conf.encoding.fallbacks
-        gaupol.gtk.conf.encoding.fallbacks.append(encoding)
+        if response != gtk.RESPONSE_OK: return
+        if encoding is None: return
+        if encoding in self.conf.fallbacks: return
+        self.conf.fallbacks.append(encoding)
         self._reload_tree_view()
         self._tree_view.grab_focus()
         store = self._tree_view.get_model()
         self._tree_view.set_cursor(len(store) - 1)
 
+    def _on_auto_check_toggled_require(self, check_button):
+        assert gaupol.util.chardet_available()
+
     def _on_auto_check_toggled(self, check_button):
         """Save the encoding auto-detection usage."""
 
-        gaupol.gtk.conf.encoding.try_auto = check_button.get_active()
+        self.conf.try_auto = check_button.get_active()
+
+    def _on_down_button_clicked_require(self, *args):
+        row = self._get_selected_row()
+        store = self._tree_view.get_model()
+        assert row < len(store) - 1
 
     def _on_down_button_clicked(self, *args):
         """Move the selected fallback encoding down."""
 
         row = self._get_selected_row()
-        encodings = gaupol.gtk.conf.encoding.fallbacks
+        encodings = self.conf.fallbacks
         encodings.insert(row + 1, encodings.pop(row))
         self._reload_tree_view()
         self._tree_view.grab_focus()
@@ -220,25 +227,30 @@ class _FilePage(gaupol.Delegate):
     def _on_locale_check_toggled(self, check_button):
         """Save the locale encoding usage."""
 
-        gaupol.gtk.conf.encoding.try_locale = check_button.get_active()
+        self.conf.try_locale = check_button.get_active()
+
+    def _on_remove_button_clicked_require(self, *args):
+        self._get_selected_row() is not None
 
-    @gaupol.gtk.util.asserted_return
     def _on_remove_button_clicked(self, *args):
         """Remove the selected encoding."""
 
         row = self._get_selected_row()
-        gaupol.gtk.conf.encoding.fallbacks.pop(row)
+        self.conf.fallbacks.pop(row)
         self._reload_tree_view()
         self._tree_view.grab_focus()
         store = self._tree_view.get_model()
-        assert len(store) > 0
+        if len(store) <= 0: return
         self._tree_view.set_cursor(max(row - 1, 0))
 
+    def _on_up_button_clicked_require(self, *args):
+        self._get_selected_row() > 0
+
     def _on_up_button_clicked(self, *args):
         """Move the selected encoding up."""
 
         row = self._get_selected_row()
-        encodings = gaupol.gtk.conf.encoding.fallbacks
+        encodings = self.conf.fallbacks
         encodings.insert(row - 1, encodings.pop(row))
         self._reload_tree_view()
         self._tree_view.grab_focus()
@@ -249,8 +261,8 @@ class _FilePage(gaupol.Delegate):
 
         store = self._tree_view.get_model()
         store.clear()
-        for encoding in gaupol.gtk.conf.encoding.fallbacks:
-            store.append([gaupol.encodings.code_to_long_name(encoding)])
+        for encoding in self.conf.fallbacks:
+            store.append((gaupol.encodings.code_to_long_name(encoding),))
         self._set_sensitivities()
 
     def _set_sensitivities(self):
@@ -273,7 +285,9 @@ class _PreviewPage(gaupol.Delegate):
         get_widget = self._glade_xml.get_widget
         self._app_combo = get_widget("preview_app_combo")
         self._command_entry = get_widget("preview_command_entry")
+        self._force_utf_8_check = get_widget("preview_force_utf_8_check")
         self._offset_spin = get_widget("preview_offset_spin")
+        self.conf = gaupol.gtk.conf.preview
 
         self._init_app_combo()
         self._init_values()
@@ -283,74 +297,77 @@ class _PreviewPage(gaupol.Delegate):
         """Initialize the application combo box."""
 
         store = self._app_combo.get_model()
-        for name in gaupol.gtk.VIDEO_PLAYER.labels:
-            store.append([name])
-        store.append([gaupol.gtk.COMBO_SEPARATOR])
-        store.append([_("Custom")])
+        for label in (x.label for x in gaupol.players):
+            store.append((label,))
+        store.append((gaupol.gtk.COMBO_SEPARATOR,))
+        store.append((_("Custom"),))
         function = gaupol.gtk.util.separate_combo
         self._app_combo.set_row_separator_func(function)
 
     def _init_signal_handlers(self):
         """Initialize signal handlers."""
 
-        gaupol.gtk.util.connect(self, "_app_combo", "changed")
-        gaupol.gtk.util.connect(self, "_command_entry", "changed")
-        gaupol.gtk.util.connect(self, "_offset_spin", "value-changed")
+        gaupol.util.connect(self, "_app_combo", "changed")
+        gaupol.util.connect(self, "_command_entry", "changed")
+        gaupol.util.connect(self, "_force_utf_8_check", "toggled")
+        gaupol.util.connect(self, "_offset_spin", "value-changed")
 
     def _init_values(self):
         """Initialize default values for widgets."""
 
-        self._offset_spin.set_value(gaupol.gtk.conf.preview.offset)
-        if not gaupol.gtk.conf.preview.use_custom:
-            player = gaupol.gtk.conf.preview.video_player
+        if not self.conf.use_custom:
+            player = self.conf.video_player
             self._app_combo.set_active(player)
-            self._command_entry.set_text(player.command)
-            return self._command_entry.set_editable(False)
-        store = self._app_combo.get_model()
-        self._app_combo.set_active(len(store) - 1)
-        command = gaupol.gtk.conf.preview.custom_command
+        else: # Use custom command.
+            store = self._app_combo.get_model()
+            self._app_combo.set_active(len(store) - 1)
+        command = gaupol.gtk.util.get_preview_command()
         self._command_entry.set_text(command)
-        self._command_entry.set_editable(True)
+        self._command_entry.set_editable(self.conf.use_custom)
+        self._force_utf_8_check.set_active(self.conf.force_utf_8)
+        self._offset_spin.set_value(self.conf.offset)
 
     def _on_app_combo_changed(self, combo_box):
         """Save the video player and show it's command."""
 
         index = combo_box.get_active()
-        if index in gaupol.gtk.VIDEO_PLAYER.members:
-            gaupol.gtk.conf.preview.use_custom = False
-            player = gaupol.gtk.VIDEO_PLAYER.members[index]
-            gaupol.gtk.conf.preview.video_player = player
-            self._command_entry.set_text(player.command)
-            return self._command_entry.set_editable(False)
-        gaupol.gtk.conf.preview.use_custom = True
-        command = gaupol.gtk.conf.preview.custom_command
+        self.conf.use_custom = not (index in gaupol.players)
+        if index in gaupol.players:
+            player = gaupol.players[index]
+            self.conf.video_player = player
+        command = gaupol.gtk.util.get_preview_command()
         self._command_entry.set_text(command)
-        self._command_entry.set_editable(True)
+        self._command_entry.set_editable(self.conf.use_custom)
 
-    @gaupol.gtk.util.asserted_return
     def _on_command_entry_changed(self, entry):
         """Save the custom command."""
 
-        assert gaupol.gtk.conf.preview.use_custom
-        gaupol.gtk.conf.preview.custom_command = entry.get_text()
+        if not self.conf.use_custom: return
+        self.conf.custom_command = entry.get_text()
+
+    def _on_force_utf_8_check_toggled(self, check_button):
+        """Save forced UTF-8 preview setting."""
+
+        self.conf.force_utf_8 = check_button.get_active()
+        command = gaupol.gtk.util.get_preview_command()
+        self._command_entry.set_text(command)
 
     def _on_offset_spin_value_changed(self, spin_button):
         """Save the start position offset."""
 
-        gaupol.gtk.conf.preview.offset = spin_button.get_value()
+        self.conf.offset = spin_button.get_value()
 
 
-class PreferencesDialog(GladeDialog):
+class PreferencesDialog(gaupol.gtk.GladeDialog):
 
     """Dialog for editing preferences."""
 
-    def __init__(self):
+    def __init__(self, parent):
 
-        GladeDialog.__init__(self, "preferences-dialog")
+        gaupol.gtk.GladeDialog.__init__(self, "preferences.glade")
         self._editor_page = _EditorPage(self)
         self._file_page = _FilePage(self)
         self._preview_page = _PreviewPage(self)
 
-        self.set_transient_for(None)
-        self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_NORMAL)
+        self.set_transient_for(parent)
         self.set_default_response(gtk.RESPONSE_CLOSE)
