@@ -9,14 +9,14 @@
 #
 # Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along with
-# Gaupol.  If not, see <http://www.gnu.org/licenses/>.
+# Gaupol. If not, see <http://www.gnu.org/licenses/>.
 
 """Cell renderer widgets for subtitle data."""
 
 from .multiline import MultilineCellRenderer
-from .time import TimeCellRenderer
+from .time      import TimeCellRenderer
 
-__all__ = ["MultilineCellRenderer", "TimeCellRenderer"]
+__all__ = ("MultilineCellRenderer", "TimeCellRenderer",)
