@@ -9,20 +9,18 @@
 #
 # Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along with
-# Gaupol.  If not, see <http://www.gnu.org/licenses/>.
+# Gaupol. If not, see <http://www.gnu.org/licenses/>.
 
 import gtk
-
-from gaupol.gtk import unittest
-from .. import entries
+import gaupol.gtk
 
 
-class TestTimeEntry(unittest.TestCase):
+class TestTimeEntry(gaupol.gtk.TestCase):
 
-    def run(self):
+    def run__entry(self):
 
         self.entry.set_text("12:34:56.789")
         window = gtk.Window()
@@ -35,7 +33,7 @@ class TestTimeEntry(unittest.TestCase):
 
     def setup_method(self, method):
 
-        self.entry = entries.TimeEntry()
+        self.entry = gaupol.gtk.TimeEntry()
 
     def test_set_text(self):
 
